# 类型别名和类型定义


## 类型别名

类型别名是用来给现有类型提供一个替代的名称。它并不会创建一个新的类型，而是给现有类型起一个别名，两者在类型系统中是完全相同的。类型别名的声明使用 `type` 关键字，并且使用赋值符号（`=`）来定义别名。

如：`type MyInt = int` 是类型别名（type alias），它创建了一个已存在类型的别名。这意味着 `MyInt` 和 `int` 是完全相同的类型，它们可以互换使用，没有任何区别。这个功能主要用于提高代码的可读性，尤其是在代码中需要使用相同类型的不同命名时。例如：

```go
type MyInt = int

func main() {
    var a MyInt
    var b int
    a = 10
    b = 5

    fmt.Println(a + b) // 可以正常运行，因为 MyInt 和 int 是相同的类型
}
```

## 类型定义

类型定义是创建一个新的类型，这个新类型和原类型在类型系统中是不同的。通过类型定义，我们可以为已有的类型创建一个新的名称，并且可以在需要时对其进行扩展或限制。类型定义的声明也使用 `type` 关键字，但是不使用赋值符号，而是直接定义新类型。

如：`type MyInt int` 是创建了一个新的类型 `MyInt`，它基于 `int` 类型，但是在类型系统中它们是不同的。这种方式定义的类型是独立的类型，不能直接和其它类型进行运算。需要显式地进行类型转换。例如：

```go
type MyInt int

func main() {
    var a MyInt
    var b int
    a = 10
    b = 5

    fmt.Println(a + b) // 不能编译通过，因为 MyInt 和 int 是不同的类型
}
```

总的来说，`type MyInt = int` 是类型别名，而 `type MyInt int` 是创建了一个新的类型。在大多数情况下，应该使用类型别名来提高代码的可读性，而不是创建新的类型。

## 区别总结

- **类型别名**：给现有类型提供一个替代的名称，不会创建新类型。使用 `=` 来声明。可以与原类型互换使用，两者是相同的类型。
- **类型定义**：创建一个新的类型，它和原类型在类型系统中是不同的。使用新名称直接声明。创建了一个新的类型，不能直接与原类型互换使用，需要进行显式的类型转换。

---

> 作者: [Hollis](https://www.xuexiqu.cn/)  
> URL: https://www.xuexiqu.cn/golang/interview/type_alias_and_type_definition/  

